#include "GstPlayer.h"

GstPlayer::GstPlayer()
{
	gst_init(NULL, NULL);
	init_pipeline();
}

GstPlayer::~GstPlayer()
{
}

void
GstPlayer::play()
{
	gst_element_set_state(pipeline, GST_STATE_PLAYING);
}

void
GstPlayer::pause()
{
	gst_element_set_state(pipeline, GST_STATE_PAUSED);
}

void
GstPlayer::stop()
{
	destroy();
	init_pipeline();
}

void
GstPlayer::destroy()
{
	gst_element_set_state(pipeline, GST_STATE_NULL);
	gst_object_unref(pipeline);
	gst_object_unref(bus);
}

GstBus*
GstPlayer::get_bus()
{
	return bus;
}

void
GstPlayer::init_pipeline()
{
	pipeline = gst_pipeline_new("Demo pipeline");

	GstElement *src, *demux, *decoder, *sink;

	src = gst_element_factory_make(DEFAULT_SRC, "src");
	demux = gst_element_factory_make(DEFAULT_DEMUXER, "demux");
	decoder = gst_element_factory_make(DEFAULT_DECODER, "decoder");
	sink = gst_element_factory_make(DEFAULT_SINK, "sink");

	g_object_set(G_OBJECT(src), "location", DEFAULT_VIDEO_LOCATION, NULL);
	g_object_set(G_OBJECT(decoder), "fmt", 1, NULL);

	if(!src || !demux || !decoder || !sink){
		g_warning("One of elements failed to initialize");
		exit(0);
	}

	gst_bin_add_many(GST_BIN(pipeline), src, demux, decoder, sink, NULL);
	if(!pipeline){
		g_warning("Error adding elements to pipeline");
		exit(0);
	}

	if(!gst_element_link(src, demux))
	{
		g_warning("\nFailed to link video source to demuxer. Now exiting");
		exit(-1);
	}

	if(!gst_element_link(decoder, sink))
	{
		g_warning("\nFailed to link video decoder to sink. Now exiting");
		exit(-1);
	}

	/*
	GstCaps *src_demux_caps;
	src_demux_caps = gst_caps_new_simple("video/quicktime", NULL);
	if(!gst_element_link_filtered(src, demux, src_demux_caps)){
		g_warning("Link src to demux failed");
		exit(0);
	}
	gst_caps_unref(src_demux_caps);

	GstCaps *decoder_sink_caps;
	decoder_sink_caps = gst_caps_new_simple("video/x-raw-yuv",
											"format", GST_TYPE_FOURCC, GST_MAKE_FOURCC('I','4','2','0'),
											"width", G_TYPE_INT, 1280,
											"height", G_TYPE_INT, 720, NULL);
	if(!gst_element_link_filtered(decoder, sink, decoder_sink_caps)){
		g_warning("Link decoder to sink failed");
		exit(0);
	}
	gst_caps_unref(decoder_sink_caps);
	*/

	g_signal_connect(demux, "pad-added", G_CALLBACK(on_pad_added), decoder);

	bus = gst_pipeline_get_bus(GST_PIPELINE(pipeline));

	gst_element_set_state(pipeline, GST_STATE_READY);
}

void
on_pad_added(GstElement *src, GstPad *src_pad, gpointer user_data)
{
	// If pad added is video pad, link to decoder
	if(strcmp(gst_pad_get_name(src_pad), "video_00") == 0)
	{
		GstElement *sink_element = (GstElement*) user_data;
		GstPad *sink_pad = gst_element_get_static_pad(sink_element, "sink");

		/*
		GstCaps *caps;
		caps = gst_caps_new_simple("video/mpeg", NULL);

		gst_pad_set_caps(src_pad, caps);
		*/

		g_print("Attempting to link");
		if(gst_pad_link(src_pad, sink_pad) != GST_PAD_LINK_OK){
			g_warning("Failed dynamic linking attempt");
			// exit(0);
		}
		gst_object_unref(sink_pad);

		// gst_caps_unref(caps);
	}
}

